# Copyright (c) 2021 ZouMingzhe <zoumingzhe@qq.com>
# This module is part of the MyPhotos package, which is released under a MIT licence.

"""
for config
"""
from .config.selfconfig     import (selfconfig)


"""
for data
"""
from .data.fbasic           import (fbasic)
from .data.hash             import (hash)
from .data.xls              import (xls)
